@@ -7,34 +7,44 @@ Log::Handler::Output::DBI - Log messages to a database.
     use Log::Handler::Output::DBI;
 
     my $db = Log::Handler::Output::DBI->new(
-        # database connection
-        database   => 'database',
-        driver     => 'mysql',
-        user       => 'user',
-        password   => 'password',
-        host       => '127.0.0.1',
-        port       => 3306,
+        # database source
+        database    => 'database',
+        driver      => 'mysql',
+        host        => '127.0.0.1',
+        port        => 3306,
+
+        # or with "dbname" instead "database"
+        dbname      => 'database',
+        driver      => 'Pg',
+        host        => '127.0.0.1',
+        port        => 5432,
+
+        # or with data_source
+        data_source => 'dbi:mysql:database=database;host=127.0.0.1;port=3306',
+
+        # Username and password
+        user        => 'user',
+        password    => 'password',
 
         # debugging
-        debug      => 1,
+        debug       => 1,
 
         # table, columns and values (as string)
-        table      => 'messages',
-        columns    => 'level, ctime, cdate, pid, hostname, progname, message',
-        values     => '%level, %time, %date, %pid, %hostname, %progname, %message',
+        table       => 'messages',
+        columns     => 'level ctime cdate pid hostname progname message',
+        values      => '%level %time %date %pid %hostname %progname %message',
 
         # table, columns and values (as array reference)
-        table      => 'messages',
-        columns    => [ qw/level ctime cdate pid hostname progname message/ ],
-        values     => [ qw/%level %time %date %pid %hostname %progname %message/ ],
+        table       => 'messages',
+        columns     => [ qw/level ctime cdate pid hostname progname message/ ],
+        values      => [ qw/%level %time %date %pid %hostname %progname %message/ ],
 
         # table, columns and values (your own statement)
-        statement  => 'insert into messages (level,ctime,cdate,pid,hostname,progname,message) values (?,?,?,?,?,?,?)',
-        values     => [ qw/%level %time %date %pid %hostname %progname %message/ ],
+        statement   => 'insert into messages (level,ctime,cdate,pid,hostname,progname,message) values (?,?,?,?,?,?,?)',
+        values      => [ qw/%level %time %date %pid %hostname %progname %message/ ],
 
         # if you like persistent connections and want to re-connect
-        persistent => 1,
-        reconnect  => 1,
+        persistent  => 1,
     );
 
     my %message = (
@@ -51,7 +61,7 @@ Log::Handler::Output::DBI - Log messages to a database.
 
 =head1 DESCRIPTION
 
-With this output you can insert messages to a database table.
+With this output you can insert messages into a database table.
 
 =head1 METHODS
 
@@ -125,7 +135,7 @@ With this option you have to set the values for the insert.
         values => [ qw/%level %time %date %pid %hostname %progname %message/ ],
 
 The placeholders are identical with the pattern names that you have to pass
-with the option C<message_pattern>.
+with the option C<message_pattern> from L<Log::Handler>.
 
     %L   level
     %T   time
@@ -133,27 +143,28 @@ with the option C<message_pattern>.
     %P   pid
     %H   hostname
     %N   newline
-    %R   runtime
     %C   caller
     %p   package
     %f   filename
     %l   line
     %s   subroutine
     %S   progname
+    %r   runtime
     %t   mtime
     %m   message
 
 Take a look to the documentation of L<Log::Handler> for all possible patterns.
 
-=item B<persistent> and B<reconnect>
+=item B<persistent>
 
 With this option you can enable or disable a persistent database connection and
 re-connect if the connection was lost.
 
-If you use persistent connections then a ping is send to the database before
-each insert. If the ping fails then a re-connect is executed.
+This option is set to 1 on default.
 
-Both options are set to 1 on default.
+=item B<reconnect>
+
+C<reconnect> is deprecated!
 
 =item B<dbi_params>
 
@@ -194,7 +205,6 @@ Log a message to the database.
         columns    => [ qw/level ctime message/ ],
         values     => [ qw/%level %time %message/ ],
         persistent => 1,
-        reconnect  => 1,
     );
 
     $db->log(
@@ -230,21 +240,15 @@ No exports.
 
 Please report all bugs to <jschulz.cpan(at)bloonix.de>.
 
+If you send me a mail then add Log::Handler into the subject.
+
 =head1 AUTHOR
 
 Jonny Schulz <jschulz.cpan(at)bloonix.de>.
 
-=head1 QUESTIONS
-
-Do you have any questions or ideas?
-
-MAIL: <jschulz.cpan(at)bloonix.de>
-
-If you send me a mail then add Log::Handler into the subject.
-
 =head1 COPYRIGHT
 
-Copyright (C) 2007 by Jonny Schulz. All rights reserved.
+Copyright (C) 2007-2008 by Jonny Schulz. All rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -255,23 +259,20 @@ package Log::Handler::Output::DBI;
 
 use strict;
 use warnings;
-our $VERSION = '0.00_08';
-our $ERRSTR  = '';
-
 use Carp;
 use DBI;
 use Params::Validate;
 
+our $VERSION = '0.05';
+our $ERRSTR  = '';
+
 sub new {
     my $class   = shift;
     my $options = $class->_validate(@_);
     my $self    = bless $options, $class;
 
-    warn "Create a new Log::Handler::Output::DBI object" if $self->{debug};
-
-    if ($self->{persistent}) {
-        warn "Peristent connections is set to true" if $self->{debug};
-        $self->connect or return undef;
+    if ($self->{debug}) {
+        warn "Create a new Log::Handler::Output::DBI object";
     }
 
     return $self;
@@ -280,40 +281,27 @@ sub new {
 sub log {
     my $self    = shift;
     my $message = @_ > 1 ? {@_} : shift;
-
-    if ($self->{persistent}) {
-        warn "ping the database" if $self->{debug};
-        if (!$self->{dbh}->ping) {
-            $self->connect or return undef;
-        }
-    } else {
-        $self->connect or return undef;
-    }
-
-    warn "prepare: $self->{statement}" if $self->{debug};
-
-    my $sth = $self->{dbh}->prepare($self->{statement})
-        or return $self->_raise_error("DBI prepare error: ".$self->{dbh}->errstr);
-
-    my @values;
+    my @values  = ();
 
     foreach my $v (@{$self->{values}}) {
         if (ref($v) eq 'CODE') {
             push @values, &$v();
-        } elsif ($v =~ /^%(.+)/) {
+        } elsif ($v =~ /^%(.+)/ && exists $message->{$1}) {
             push @values, $message->{$1};
         } else {
             push @values, $v;
         }
     }
 
-    warn "execute: ".@values." bind values" if $self->{debug};
+    if ($self->{debug}) {
+        warn "execute: ".@values." bind values";
+    }
 
-    $sth->execute(@values)
-        or return $self->_raise_error("DBI execute error: ".$sth->errstr);
+    $self->connect or return undef;
 
-    $sth->finish
-        or return $self->_raise_error("DBI finish error: ".$sth->errstr);
+    if ( ! $self->{sth}->execute(@values) ) {
+        return $self->_raise_error("DBI execute error: ".DBI->errstr);
+    }
 
     if (!$self->{persistent}) {
         $self->disconnect or return undef;
@@ -325,22 +313,49 @@ sub log {
 sub connect {
     my $self = shift;
 
-    warn "Connect to the database: $self->{cstr}->[0] ..." if $self->{debug};
+    if ($self->{persistent} && $self->{dbh}) {
+        eval { $self->{dbh}->do("select 1") or die DBI->errstr };
+        return 1 unless $@;
+    }
+
+    if ($self->{debug}) {
+        warn "Connect to the database: $self->{cstr}->[0] ...";
+    }
+
 
     my $dbh = DBI->connect(@{$self->{cstr}})
         or return $self->_raise_error("DBI connect error: ".DBI->errstr);
 
+    my $sth = $dbh->prepare($self->{statement})
+        or return $self->_raise_error("DBI prepare error: ".$dbh->errstr);
+
     $self->{dbh} = $dbh;
+    $self->{sth} = $sth;
 
     return 1;
 }
 
 sub disconnect {
     my $self = shift;
-    warn "Disconnect from database" if $self->{debug};
-    $self->{dbh}->disconnect
-        or return $self->_raise_error("DBI disconnect error: ".DBI->errstr);;
-    delete $self->{dbh};
+
+    if ($self->{sth}) {
+        $self->{sth}->finish
+            or return $self->_raise_error("DBI finish error: ".$self->{sth}->errstr);
+
+        delete $self->{sth};
+    }
+
+    if ($self->{dbh}) {
+        if ($self->{debug}) {
+            warn "Disconnect from database";
+        }
+
+        $self->{dbh}->disconnect
+            or return $self->_raise_error("DBI disconnect error: ".DBI->errstr);;
+
+        delete $self->{dbh};
+    }
+
     return 1;
 }
 
@@ -354,11 +369,21 @@ sub _validate {
     my $class = shift;
 
     my %options = Params::Validate::validate(@_, {
+        data_source => {
+            type => Params::Validate::SCALAR,
+            optional => 1,
+        },
         database => {
             type => Params::Validate::SCALAR,
+            optional => 1,
+        },
+        dbname => {
+            type => Params::Validate::SCALAR,
+            optional => 1,
         },
         driver => {
             type => Params::Validate::SCALAR,
+            optional => 1,
         },
         user => {
             type => Params::Validate::SCALAR,
@@ -366,12 +391,11 @@ sub _validate {
         },
         password => {
             type => Params::Validate::SCALAR,
-            depends => [ 'user' ],
+            optional => 1,
         },
         host => {
             type => Params::Validate::SCALAR,
             optional => 1,
-            depends => [ 'port' ],
         },
         port => {
             type => Params::Validate::SCALAR,
@@ -394,11 +418,11 @@ sub _validate {
             type => Params::Validate::SCALAR,
             optional => 1,
         },
-        reconnect => {
+        persistent => {
             type => Params::Validate::SCALAR,
             default => 1,
         },
-        persistent => {
+        reconnect => { # not used any more
             type => Params::Validate::SCALAR,
             default => 1,
         },
@@ -417,19 +441,33 @@ sub _validate {
         Carp::croak "Missing one of the mandatory options: 'statement' or 'table' and 'columns'";
     }
 
-    # build the connect string
-    my @cstr = ("dbi:$options{driver}:database=$options{database}");
+    # build the connect string (data source name)
+    my @cstr = ();
 
-    if ($options{host}) {
-        $cstr[0] .= ";host=$options{host}";
-        if ($options{port}) {
-            $cstr[0] .= ";port=$options{port}";
+    if (defined $options{data_source}) {
+        @cstr = ($options{data_source});
+    } elsif ($options{driver} && ($options{database} || $options{dbname})) {
+        $cstr[0] = "dbi:$options{driver}:";
+
+        if ($options{database}) {
+            $cstr[0] .= "database=$options{database}";
+        } else {
+            $cstr[0] .= "dbname=$options{dbname}";
         }
+
+        if ($options{host}) {
+            $cstr[0] .= ";host=$options{host}";
+            if ($options{port}) {
+                $cstr[0] .= ";port=$options{port}";
+            }
+        }
+    } else {
+        Carp::croak "Missing mandatory options data_source or database/dbname";
     }
 
     if ($options{user}) {
         $cstr[1] = $options{user};
-        if ($options{port}) {
+        if ($options{password}) {
             $cstr[2] = $options{password};
         }
     }
@@ -440,7 +478,7 @@ sub _validate {
     # build the statement
 
     if (!ref($options{values})) {
-        $options{values} = [ split /\s*,\s*/, $options{values} ];
+        $options{values} = [ split /[\s,]+/, $options{values} ];
     }
 
     if (!$options{statement}) {
@@ -450,7 +488,7 @@ sub _validate {
         if (ref($options{columns})) {
             $options{statement} .= join(',', @{$options{columns}});
         } else {
-            $options{statement} .= $options{columns};
+            $options{statement} .= join(',', split /[\s,]+/, $options{columns});
         }
 
         $options{statement} .= ') values (';
